/*****************************************************************************
 *
 * File ..................: conio.c
 * Purpose ...............: Keyboard I/O Routines
 * Last modification date : 15-Mar-2001
 *
 *****************************************************************************
 * Copyright (C) 2001
 *
 * Darryl Perry		FIDO:		1:211/105
 * Sacramento, CA.
 * USA
 *
 * This file is part of TopDoor/LNX.
 *
 * This utility is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * TopDoor/LNX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along TopDoor/LNX; see the file COPYING.  If not, write to the
 * Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stropts.h>
#include <sys/ioctl.h>
#include <stdarg.h>
#include <termios.h>
#include <string.h>
#include <sys/poll.h>
#include <time.h>
#include "conio.h"
#include "basics.h"
#include "mci.h"

#define echo_on()  term_option(0)
#define echo_off() term_option(1)

char peekch=0;

void od_input_str(char *sStr, int iMaxlen, unsigned char hichar, unsigned char lochar) 
{
	int i,ok;
	char ch=0,prevchar,dispch;

	printf("%s",sStr);
	for(i=strlen(sStr);i<iMaxlen;i++)
		printf(" ");
	for(;i>strlen(sStr);i--)
		printf("\b");
	i=strlen(sStr);
	open_keyboard();
	do{
		ok=FALSE;
		prevchar=ch;
		fflush(stdout);
		ch=getkey1();
		dispch=ch;

		if(ch=='\b' || ch==127){
			if(i>0){
				printf("\b \b");
				i--;
				prevchar=*sStr;
				dispch='\0';
				}
			ok=FALSE;
			}
		else{
			if(ch>=hichar && ch<lochar){
				prevchar=sStr[i];
				if(ch!='\n' && ch!='\r' && ch!='\x1b' && ch!='\b' && ch!=127)
					sStr[i]=ch;
				else{
					sStr[i]='\0';
					}
				printf("%c",sStr[i]);
				i++;
				}	
			}
	}while(ch!=10 && i<=iMaxlen);
	sStr[i]='\0';
	close_keyboard();
}

char od_get_answer(char *alist)
{
	int done=FALSE;
	char ch,ch1;

	while(!done){
		ch=od_get_key(TRUE);
		if(ch>='a' && ch<='z') 
			ch-=32;
		if(strchr(alist,ch)){
			ch1=ch;
			done=TRUE;
			}
		}
/*	od_putch(ch1); */
	return(ch1);
}


void od_putch(char chToDisplay)
{
	od_printf("%c",chToDisplay);
}

BOOL od_send_file(char *filename)
{
	FILE *fptr;
	char ttxt[161];
	BOOL retval=FALSE;

	fptr=fopen(filename,"rt");
	if(fptr==NULL){
		nl();
		od_printf("Unable to print %s",filename);nl();
		paws();
		}	
	else{
		retval=TRUE;
		while(fgets(ttxt,161,fptr)!=NULL){
			od_printf(ttxt);
			}
	}
	return retval;
}


void od_printf(char *fmt, ...)
{
	va_list argptr;
	char buffer[250];

	va_start(argptr, fmt);
	vsprintf(buffer, fmt, argptr);
	va_end(argptr);
/*
 *	printf(buffer); 
 */

	printf(buffer);
}

void od_clr_scr()
{
	od_printf("\x1b[2J\x1b[1;1H");
}


void od_repeat(char chValue, BYTE btTimes)
{
	int l1;

	for(l1=0;l1<btTimes;l1++)
		od_putch(chValue);
}


void od_exit(int exitlev, BOOL lo)
{
	exit(exitlev);
}

#if defined (UNIX)
int strnicmp(char *in, char *out, int len)
{
	int i1,x=FALSE;

	for(i1=0;i1<len;i1++){
		if(in[i1]!=out[i1] && !x)
			x=TRUE;
		}
	return(x);
}
#endif


/*
**  Terminal functions
**
**  Option 0 - Turns the ECHO on.
**         1 - Turns the ECHO off.
**         2 - Waits forever for keyboard activity.
**
**  Public Domain *IX terminal functions.
**  Slung together by Steve Poole for RBS's SNIPPETS.
**
*/

int term_option(int option)
{
	struct termios attributes;

    switch(option) {
      case 0:
             if(tcgetattr(STDIN_FILENO,&attributes) != 0) return (-1);
             attributes.c_lflag |= ECHO;
             attributes.c_lflag |= ICANON;
             if(tcsetattr(STDIN_FILENO,TCSANOW,&attributes) != 0) return (-1);
             break;
      case 1:
             if(tcgetattr(STDIN_FILENO,&attributes) != 0) return (-1);
             attributes.c_lflag &= ~(ECHO);
             if(tcsetattr(STDIN_FILENO,TCSAFLUSH,&attributes) != 0) return (-1);
             break;
      case 2:
             if(tcgetattr(STDIN_FILENO,&attributes) != 0) return (-1);
             attributes.c_lflag    &= ~(ICANON);
             attributes.c_cc[VMIN]  = 1;
             attributes.c_cc[VTIME] = 1;
             if(tcsetattr(STDIN_FILENO,TCSANOW,&attributes) != 0) return (-1);
             break;
      case 3:
             if(tcgetattr(STDIN_FILENO,&attributes) != 0) return (-1);
             attributes.c_lflag    &= ~(ICANON);
             attributes.c_cc[VMIN]  = 0;
             attributes.c_cc[VTIME] = 0;
             if(tcsetattr(STDIN_FILENO,TCSANOW,&attributes) != 0) return (-1);
             break;
	default:
		printf("Error in terminal options routine, BAD OPTION %d\n",option);
        return(-1);
        break;
    }
	return(1);
}


char od_get_key(BOOL bWait)
{
	int istat, key;
	char buf[2];
	time_t One=time(NULL),Two;

	fflush(stdout);

	if (0 > term_option(1)) return EOF;
	if (0 > term_option(2)) return EOF;
	if (0 > term_option(3)) return EOF;
	do{
		istat = read(STDIN_FILENO,&buf,1);
		Two=time(NULL);
		if(difftime(Two,One)>300){
			printf("\n\r\nKeyboard inactivity has occured.\n\r");
			term_option(0);
			exit(1);
			}
		microsleep(1000);
		}while(istat==0);
	
	
	if (istat < 0) 	key = EOF;
	else  			key = (int)buf[0];
	term_option(0);
	return key;
}



void open_keyboard()
{
	term_option(1);	 
/*	term_option(2);	 */
	term_option(3);	
}

void close_keyboard()
{
	term_option(0);
}

int kbhit()
{
	int istat;
	char buf;

    istat = read(STDIN_FILENO,&buf,1);
	if(istat==1)
		peekch=buf;
	return istat;
}

int readch()
{
	char ch;
	if(peekch!=-1){
		ch=peekch;
		peekch=-1;
		}
	else{
		fflush(stdout);
		read(0,&ch,1);
		}
	return ch;
}

int getkey1()
{
	time_t begintime,testtime;

	(void) time(&begintime);
	
	while(!kbhit()){
		(void) time(&testtime);
		if(difftime(begintime,testtime)>300){
			printf("\n\n\rKeyboard inactivity has occured.\n\r\n");
			exit(1);
			}
		microsleep(1000);
		}
	fflush(stdout);
	return(readch());
}

void microsleep(unsigned int microsecs)
{
	struct pollfd dummy;
	int timeout;
	if((timeout=microsecs/1000)<=0)
		timeout=1;
	poll(&dummy,0,timeout);
}
