/*****************************************************************************
 *
 * File ..................: topdoor.c
 * Purpose ...............: Main file
 * Last modification date : 15-Mar-2001
 *
 *****************************************************************************
 * Copyright (C) 2001
 *
 * Darryl Perry		FIDO:		1:211/105
 * Sacramento, CA.
 * USA
 *
 * This file is part of TopDoor/LNX.
 *
 * This utility is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * TopDoor/LNX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along TopDoor/LNX; see the file COPYING.  If not, write to the
 * Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include "topdoor.h"
#include "conio.h"
#include "mci.h"


void resetall();
void ansiout();
void asciout();
int savedatatmp(gamelistrec *gd, int recno);
void delete(int recno);
void sortem();
void usage();

char xst[91],ansipath[91],ascipath[91],pawsstr[41];
extern char *optarg;
extern int optind, opterr, optopt;

int main(int argc, char *argv[])
{
	int opt,num=0,cnt=0,srtflg=TRUE;

	mci("~SCTopdoor/LNX v1.0, The Door Game Usage Status Maker");
	mci("~SMCopyright 2001 by Darryl Perry~SM");

	/* Find out where the work dir is */
	strcpy(workdir,argv[0]);
	dirname(workdir);

	/* Set up the defaults */
	sprintf(ansipath,"%stopdoor.ans",workdir);
	sprintf(ascipath,"%stopdoor.asc",workdir);
	strcpy(pawsstr,"|PA");

    /* If there is no topdoor.dat file, go to the editor */
	if(!readdata(&game,0))	{
		editor();
		exit(0);
		}

	/* Get the options */
	while((opt=getopt(argc,argv,"not:a:s:p:er"))!=-1){
		cnt++;
		switch(opt){
			case 'o':	break;
			case 'n':	srtflg=FALSE;	break;
			case 'a':
				strcpy(ansipath,optarg);
				break;
			case 's':
				strcpy(ascipath,optarg);
				break;
			case 'p':
				strcpy(pawsstr,optarg);
				break;
			case ':':	
				printf("\n\rOption requires a value.\n\r");
				break;
			case 'e':	editor();	break;
			case 'r':	
				resetall();	
				break;
			case 't':	
				num=findrec(optarg);
				game.count++;
				savedata(&game,num);
				break;
			case '?':	
				usage();
				break;
				}
			}
	if(cnt==0){
		usage();
		}
	else{
		if(srtflg) sortem();
		ansiout();
		asciout();
		printf("\n\r");
		}
	return 0;
}

void usage()
{
	printf("\n\rUSAGE: topdoor [ -reo | -t <tag> [ -a ansifile | -s asciifile ] ]");
	printf("\n\n\r-e Envoke the editor.");
	printf("\n\r     Example: topdoor -e");
	printf("\n\n\r-o Create output files without updating the stats.");
	printf("\n\r     Example: topdoor -o");
	printf("\n\n\r-r Reset all door usage counts to zero.");
	printf("\n\r     Example: topdoor -r");
	printf("\n\n\r-t Specify the TAG of the count to increments.");
	printf("\n\r     Example: topdoor -t <tag> ");
	printf("\n\n\r-a Define the ANSI output file");
	printf("\n\r     Example: topdoor -a /path/to/ansi/file.ext");
	printf("\n\n\r-s Define the ASCII output file");
	printf("\n\r     topdoor -s /path/to/ascii/file.ext\n\r");
}

int findrec(char *tag)
{
	int i;
	i=0;
	while(readdata(&game,i)){
		if(strcmp(game.tag,tag)==0){
			return(i);
			}
		i++;
		}
	return(-1);
}

int readdata(gamelistrec *gd, int recno)
{
	FILE *fptr;
	char path[91];
	int x=FALSE;

	sprintf(path,"%stopdoor.dat",workdir);
	fptr=fopen(path,"rb");
	if(fptr==NULL){
		return(x);
		}
	if(fseek(fptr,(long) recno * sizeof(gamelistrec),SEEK_SET)==0){
		x=fread(gd,sizeof(gamelistrec),1,fptr);
		}
	fclose(fptr);
	return(x);
}

int savedatatmp(gamelistrec *gd, int recno)
{
	FILE *fptr;
	char path[91];
	int x=FALSE;

	sprintf(path,"%stopdoor.dat.tmp",workdir);
	fptr=fopen(path,"rb+");
	if(fptr==NULL){
		fptr=fopen(path,"wb+");
		if(fptr==NULL){
			mci("~SMUnable to write to %s~SM~SP",path);
			od_get_key(TRUE);
			return (FALSE);
			}
		}
	if(fseek(fptr,(long) recno * sizeof(gamelistrec),SEEK_SET)==0){
		x=fwrite(gd,sizeof(gamelistrec),1,fptr);
		}
	fclose(fptr);
	return(x);
}


int savedata(gamelistrec *gd, int recno)
{
	FILE *fptr;
	char path[91];
	int x=FALSE;

	sprintf(path,"%stopdoor.dat",workdir);
	fptr=fopen(path,"rb+");
	if(fptr==NULL){
		fptr=fopen(path,"wb+");
		if(fptr==NULL){
			mci("~SMUnable to write to %s~SM~SP",path);
			od_get_key(TRUE);
			return (FALSE);
			}
		}
	if(fseek(fptr,(long) recno * sizeof(gamelistrec),SEEK_SET)==0){
		x=fwrite(gd,sizeof(gamelistrec),1,fptr);
		}
	fclose(fptr);
	return(x);
}


void dirname(char *fname)
{
	int i;

	for(i=strlen(fname)-1;i>0;i--){
		if(fname[i]=='/'){
			fname[i+1]='\0';
			return;
			}
		}
}

void listgames(int page)
{
	int i,recno=(page*15);

	for(i=0;i<15;i++){
		if(readdata(&game,i+recno)){
			mci("~SM%2d) %5d %-20s %-40s",i+1+recno,game.count,game.tag,game.display);
			}
		}
}

void modify_game(int curgame)
{
	int done=FALSE;
	char ttxt[41];

	while(!done){
		readdata(&game,curgame);
		mci("~SCEdit Game Entry %d of %d~SM",curgame+1,numgames);
		mci("~SM(A) Count   : %d",game.count);
		mci("~SM(B) Tag     : %s",game.tag);
		mci("~SM(C) Display : %s",game.display);
		mci("~SM~SMCommand (]/[=Next/Prev, A,B,C,Q=Quit): ");
		switch(od_get_answer("ABCQ[]")){
			case 'Q': done=TRUE;	break;
			case 'A':	
				mci("~SMNew Count: ");
				*ttxt='\0';
				od_input_str(ttxt,5,'0','9');
				game.count=atoi(ttxt);
				savedata(&game,curgame);
				break;
			case 'B':	
				mci("~SMNew Tag: ");
				*ttxt='\0';
				od_input_str(ttxt,20,32,127);
				if(strlen(ttxt)>0)
					strcpy(game.tag,ttxt);
				savedata(&game,curgame);
				break;
			case 'C':	
				mci("~SMNew Display: ");
				*ttxt='\0';
				od_input_str(ttxt,40,32,127);
				if(strlen(ttxt)>0)
					strcpy(game.display,ttxt);
				savedata(&game,curgame);
				break;
			case '[': 
				curgame--;
				if(curgame<0)	curgame=numgames-1;
				break;
			case ']':
				curgame++;
				if(curgame>=numgames)	curgame=0;
				break;
			}
		}
}

int countlist()
{
	int i=0;
	while(readdata(&game,i)){
		i++;
		}
	return i;
}

void add_door()
{
	game.count=0;
	strcpy(game.tag,"NONE");
	strcpy(game.display,"Nothing");
	savedata(&game,numgames);
	numgames=countlist();
}

void editor()
{
	int num,done=FALSE,page=0;
	char ch,ttxt[11];

	numgames=countlist();

	if(numgames<1){
		add_door();
		}


	while(!done){
		mci("~SCTop Door Editor ~SM");
		mci("~SM~EF=78");
		listgames(page);
		mci("~SM~EF=78~SM");
		mci("`09Command (`0F?=Help, Q=Quit`09): `0F");
		ch=od_get_answer("ADMQ?");
		switch(ch){
			case '?':
				mci("~SM~SMEditor Commands:");
				mci("~SM(A) Add a new entry         (D) Delete an entry");
				mci("~SM(M) Modify an entry         (Q) Quit the editor");
				mci("~SM~SP");
				break;
			case 'Q':	done=TRUE;	break;
			case 'A':	add_door();	break;
			case 'D':	
				mci("~SMDelete which? (1-%d): ",numgames);
				*ttxt='\0';
				od_input_str(ttxt,5,'0','9');
				if(strlen(ttxt)>0)	num=atoi(ttxt)-1;
				if(num>=0 && num < numgames) delete(num);	
				numgames=countlist();
				break;
			case 'M':		
				mci("~SMModify which entry? (1-%d): ",numgames);
				*ttxt='\0';
				od_input_str(ttxt,5,'0','9');
				if(strlen(ttxt)>0){
					num=atoi(ttxt)-1;
					}
				if(num>=0 && num < numgames){
					modify_game(num);	
					}
				break;
			}
		}
}

void resetall()
{
	int num=0;

	while(readdata(&game,num)){
		game.count=0;
		savedata(&game,num);
		num++;
		}
}

int counttotalusage()
{
	int num=0,total=0;

	while(readdata(&game,num)){
		total+=game.count;
		num++;
		}
	return total;
}

void xstrng(float pct)
{
	int i,j;

	for(i=0;i<33;i++){
		xst[i]='\0';
		}
	i=pct/3;
	for(j=0;j<i;j++){
		xst[j]=223;
		}
}

void ansiout()
{
	int i,num=0,tot=0;
	float pct=0;
	FILE *fptr;
	char ttxt[161];

	fptr=fopen(ansipath,"wt");
	if(fptr==NULL){
		return;
		}

	fputs("[2J[1;34mTopdoor/LNX v1.0, Copyright 2001 by Darryl Perry\n",fptr);

	for(i=0;i<35;i++)
		fputc(205,fptr);
	fputc(203,fptr);

	for(i=0;i<3;i++)
		fputc(205,fptr);
	fputc(203,fptr);

	for(i=0;i<4;i++)
		fputc(205,fptr);
	fputc(203,fptr);

	for(i=0;i<34;i++)
		fputc(205,fptr);
	fputs("\n",fptr);

	tot=counttotalusage();

	while(readdata(&game,num)){
		pct=(float)((float)game.count/(float)tot)*100;
		xstrng(pct);
		sprintf(ttxt,"[1;37m%-34.34s [1;34m%c[1;33m%3.3d[1;34m%c%3.0f%%[1;34m%c [1;31m%s[1;34m \n",
		game.display,186,game.count,186,pct,186,xst);	
		fputs(ttxt,fptr);
		num++;
		}

	for(i=0;i<35;i++)
		fputc(205,fptr);
	fputc(202,fptr);

	for(i=0;i<3;i++)
		fputc(205,fptr);
	fputc(202,fptr);

	for(i=0;i<4;i++)
		fputc(205,fptr);
	fputc(202,fptr);

	for(i=0;i<34;i++)
		fputc(205,fptr);
	fputs("\n",fptr);

	sprintf(ttxt,"[1;37mTotal Door Usage: [1;33m%d\n%s\n\r",tot,pawsstr);
	fputs(ttxt,fptr);
	fclose(fptr);
}

void asciout()
{
	int num=0,tot=0;
	float pct=0;
	FILE *fptr;
	char ttxt[121];

	fptr=fopen(ascipath,"wt");
	if(fptr==NULL){
		return;
		}

	fputs("\n\rTopdoor/LNX v1.0, Copyright 2001 by Darryl Perry\n\n",fptr);
	fputs("===================================+===+====+=================================\n",fptr);
	tot=counttotalusage();

	while(readdata(&game,num)){
		if(tot==0)	pct=0;
		else		pct=(float)((float)game.count/(float)tot)*100;
		xstrng(pct);
		sprintf(ttxt,"%-34.34s |%3.3d|%3.0f%%| %s\n",
		game.display,game.count,pct,xst);	
		fputs(ttxt,fptr);
		num++;
		}
	fputs("===================================+===+====+=================================\n",fptr);

	sprintf(ttxt,"Total Door Usage: %d\n%s\n\r",tot,pawsstr);
	fputs(ttxt,fptr);
	fclose(fptr);
}

void delete(int recno)
{
	int i,j=0;
	char p1[91],p2[91];

	sprintf(p1,"%stopdoor.dat",workdir);
	sprintf(p2,"%stopdoor.dat.tmp",workdir);

	numgames=countlist();

	for(i=0;i<numgames;i++){
		if(i!=recno){
			readdata(&game,i);
			savedatatmp(&game,j);
			j++;
			}
		}
	remove(p1);
	rename(p2,p1);
}

void sortem()
{
	gamelistrec g1, g2;
	int i,i1,i2,j;
	
	numgames=countlist();

	for(j=0;j<numgames;j++){
		for(i=0;i<numgames-1;i++){
			i1=i;
			i2=i+1;
			readdata(&g1,i1);
			readdata(&g2,i2);
			if(g2.count>g1.count){
				savedata(&g2,i1);
				savedata(&g1,i2);
				}	
			}
		}
}
